'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var reselect = require('reselect');
var immer = require('immer');
var react = require('react');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

/* eslint react-hooks/exhaustive-deps: 0 */
immer.enablePatches();
var useIsomorphicLayoutEffect = typeof window !== "undefined" ? react.useLayoutEffect : react.useEffect;
function toAnonFunction(arg) {
  if (typeof arg === "function") return arg;
  return function () {
    return arg;
  };
}

var incrementParam = function incrementParam(num) {
  return ++num;
};

var emptyArray = [];
var Constants = {
  IS_NOT_PRODUCTION: "development" !== "production"
};
var useUpdate = function useUpdate(persistencePlugin) {
  var _useState = react.useState(0),
      setState = _useState[1];

  react.useEffect(function () {
    if (persistencePlugin === null || persistencePlugin === void 0 ? void 0 : persistencePlugin.current.isSSR) {
      persistencePlugin.getState();
    }
  }, emptyArray);
  return react.useCallback(function () {
    return setState(incrementParam);
  }, emptyArray);
};
var getPromiseWithCallbacks = function getPromiseWithCallbacks() {
  var resolve = undefined;
  var reject = undefined;
  var promise = new Promise(function (resolvePromise, rejectPromise) {
    resolve = resolvePromise;
    reject = rejectPromise;
  });
  return {
    promise: promise,
    resolve: resolve,
    reject: reject
  };
};
var isClientSide = typeof window !== "undefined";

var connectDevTools = function connectDevTools(name) {
  try {
    if (typeof window !== "undefined" && typeof window.__REDUX_DEVTOOLS_EXTENSION__ !== "undefined") {
      var devTools = window.__REDUX_DEVTOOLS_EXTENSION__.connect({
        name: "Store " + name,
        serialize: true
      });

      return devTools;
    }
  } catch (err) {}

  return undefined;
};

function isPromise(obj) {
  return !!obj && (typeof obj === "object" || typeof obj === "function") && typeof obj.then === "function";
}

var connectPersistenceStorage = function connectPersistenceStorage(_ref) {
  var persistenceName = _ref.persistenceKey,
      produce = _ref.produce,
      _ref$debounceWait = _ref.debounceWait,
      debounceWait = _ref$debounceWait === void 0 ? 3000 : _ref$debounceWait,
      _ref$persistenceMetho = _ref.persistenceMethod,
      persistenceMethod = _ref$persistenceMetho === void 0 ? isClientSide ? window.localStorage : undefined : _ref$persistenceMetho,
      isSSR = _ref.isSSR,
      resolve = _ref.resolve,
      reject = _ref.reject;
  if (!persistenceMethod) return null;
  var isConnected = false;
  var timeout;

  var setStateFn = function setStateFn(state) {
    var possiblePromise = persistenceMethod.setItem(persistenceName, JSON.stringify(state));

    if (isPromise(possiblePromise)) {
      return possiblePromise.then(function () {
        isConnected = true;
      });
    }

    if (!isConnected) isConnected = true;
    return void 0;
  };

  var setState = function setState(state) {
    var execLater = function execLater() {
      timeout = undefined;
      setStateFn(state);
    };

    if (timeout !== undefined) clearTimeout(timeout);
    timeout = setTimeout(execLater, debounceWait);
  };

  var getState = function getState() {
    if (isConnected || !persistenceMethod) {
      resolve();
      return void 0;
    }

    try {
      var statePromise = persistenceMethod.getItem(persistenceName);

      if (isPromise(statePromise)) {
        return statePromise.then(function (state) {
          if (state == null) {
            isConnected = true;
            resolve();
            return void 0;
          }

          produce(function () {
            return _extends({}, produce(), JSON.parse(state));
          });
          isConnected = true;
          resolve();
          return void 0;
        })["catch"](reject);
      } else {
        if (statePromise == null) {
          isConnected = true;
          resolve();
          return void 0;
        }

        produce(function () {
          return _extends({}, produce(), JSON.parse(statePromise));
        });
        isConnected = true;
        resolve();
        return void 0;
      }
    } catch (err) {
      reject(err);
      return void 0;
    }
  };

  if (isSSR) {
    resolve();
  } else {
    getState();
  }

  return {
    getState: getState,
    setState: setState,
    current: {
      persistenceKey: persistenceName,
      persistenceMethod: persistenceMethod,
      isSSR: isSSR,
      isConnected: isConnected,
      debounceWait: debounceWait
    }
  };
};

/**
 *  Create **react-state-selector** global store
 *
 * @export
 * @template TStore
 * Any object, including array or complex objects like Map, Set or Classes. Even primitives would work.
 * @template THooks
 * Object that defines the custom hooks of the store.
 * @template TActions
 * Object that defines the custom actions of the store.
 * @template TAsyncActions
 * Object that defines the custom async actions of the store.
 * @param {Immutable<TStore>} initialStore
 * Initial state of the store
 * @param {{
 *     hooks?: THooks;
 *     actions?: TActions;
 *     devName?: string;
 *     devToolsInProduction?: boolean;
 *   }} [options]
 * Options object.
 *
 * @returns {{
 *   useStore: IUseStore<TStore>;
 *   produce: IProduce<TStore>;
 *   asyncProduce: IAsyncProduce<TStore>;
 *   hooks: IHooksObj<TStore, typeof options>;
 *   actions: IActionsObj<TStore, typeof options> & IAsyncActionsObj<TStore, typeof options>;
 * }}
 * Resulting object of the store.
 */

function createStore(initialStore, options) {
  var _options$storagePersi;

  {
    for (var name in options === null || options === void 0 ? void 0 : options.hooks) {
      if (name.length < 4 || name.slice(0, 3) !== "use" || name.charAt(3) === name.charAt(3).toLowerCase()) {
        throw new Error("All hooks should follow the rules of hooks for naming and \"" + name + "\" doesn't");
      }
    }
  }

  {
    if ((options === null || options === void 0 ? void 0 : options.actions) && options.asyncActions) {
      var asyncActionsKeys = Object.keys(options.asyncActions);

      for (var key in options.actions) {
        if (asyncActionsKeys.includes(key)) {
          throw new Error("All the actions and asyncActions should have different names and \"" + key + "\" exists in both objects!");
        }
      }
    }
  }

  var devTools;
  var localStoragePlugin;
  var isReady = Promise.resolve();

  if ((options === null || options === void 0 ? void 0 : options.devName) && (options.devToolsInProduction || Constants.IS_NOT_PRODUCTION)) {
    devTools = connectDevTools(options.devName);
  }

  var listeners = new Map();
  var currentStore = initialStore;

  if (devTools) {
    devTools.init(currentStore);
  }

  var useStore = function useStore() {
    var update = useUpdate(localStoragePlugin);
    useIsomorphicLayoutEffect(function () {
      var globalListener = reselect.createSelector(function (s) {
        return s;
      }, function () {
        update();
      });
      listeners.set(globalListener, null);
      return function () {
        listeners["delete"](globalListener);
      };
    }, emptyArray);
    return currentStore;
  };

  var produceObj = {
    produce: function produce(draft) {
      var _localStoragePlugin;

      if (typeof draft !== "function") return currentStore;

      if (devTools) {
        var produceFn = immer.produceWithPatches(draft);
        var produceResult = produceFn(currentStore);
        currentStore = produceResult[0];
        devTools.send({
          type: "produce",
          payload: produceResult[1]
        }, currentStore);
      } else {
        var _produceFn = immer.produce(draft);

        currentStore = _produceFn(currentStore);
      }

      listeners.forEach(function (props, listener) {
        listener(currentStore, props);
      });
      (_localStoragePlugin = localStoragePlugin) === null || _localStoragePlugin === void 0 ? void 0 : _localStoragePlugin.setState(currentStore);
      return currentStore;
    },
    asyncProduce: function (draft) {
      try {
        if (typeof draft !== "function") return Promise.resolve(currentStore);
        var storeDraft = immer.createDraft(currentStore);
        return Promise.resolve(Promise.resolve(draft(storeDraft))).then(function () {
          var _localStoragePlugin2;

          immer.finishDraft(storeDraft, function (changes) {
            if (changes.length) {
              currentStore = immer.applyPatches(currentStore, changes);

              if (devTools) {
                devTools.send({
                  type: "asyncProduce",
                  payload: changes
                }, currentStore);
              }

              listeners.forEach(function (props, listener) {
                listener(currentStore, props);
              });
            }
          });
          (_localStoragePlugin2 = localStoragePlugin) === null || _localStoragePlugin2 === void 0 ? void 0 : _localStoragePlugin2.setState(currentStore);
          return currentStore;
        });
      } catch (e) {
        return Promise.reject(e);
      }
    }
  };

  if (options === null || options === void 0 ? void 0 : (_options$storagePersi = options.storagePersistence) === null || _options$storagePersi === void 0 ? void 0 : _options$storagePersi.isActive) {
    var _getPromiseWithCallba = getPromiseWithCallbacks(),
        promise = _getPromiseWithCallba.promise,
        resolve = _getPromiseWithCallba.resolve,
        reject = _getPromiseWithCallba.reject;

    isReady = promise;
    if (typeof initialStore !== "object" || Array.isArray(initialStore)) throw new Error("For local storage persistence your store has to be an object");
    var persistenceKey = options.storagePersistence.persistenceKey || options.devName;
    if (!persistenceKey) throw new Error("You have to specify persistence key or devName");
    localStoragePlugin = connectPersistenceStorage({
      persistenceKey: persistenceKey,
      produce: produceObj.produce,
      debounceWait: options.storagePersistence.debounceWait,
      persistenceMethod: options.storagePersistence.persistenceMethod,
      isSSR: options.storagePersistence.isSSR,
      resolve: resolve,
      reject: reject
    });
  }

  var actionsObj = {};
  var asyncActionsObj = {};

  var _loop = function _loop() {
    var _Object$entries$_i = _Object$entries[_i],
        actionName = _Object$entries$_i[0],
        actionFn = _Object$entries$_i[1];

    actionsObj[actionName] = function () {
      var _localStoragePlugin3;

      var actionDraft = actionFn.apply(void 0, arguments);

      if (devTools) {
        var produceFn = immer.produceWithPatches(actionDraft);
        var produceResult = produceFn(currentStore);
        currentStore = produceResult[0];
        devTools.send({
          type: actionName,
          payload: produceResult[1]
        }, currentStore);
      } else {
        var _produceFn2 = immer.produce(actionDraft);

        currentStore = _produceFn2(currentStore);
      }

      listeners.forEach(function (props, listener) {
        listener(currentStore, props);
      });
      (_localStoragePlugin3 = localStoragePlugin) === null || _localStoragePlugin3 === void 0 ? void 0 : _localStoragePlugin3.setState(currentStore);
      return currentStore;
    };
  };

  for (var _i = 0, _Object$entries = Object.entries((options === null || options === void 0 ? void 0 : options.actions) || {}); _i < _Object$entries.length; _i++) {
    _loop();
  }

  var _loop2 = function _loop2() {
    var _Object$entries2$_i = _Object$entries2[_i2],
        actionName = _Object$entries2$_i[0],
        actionFn = _Object$entries2$_i[1];

    asyncActionsObj[actionName] = function () {
      try {
        return Promise.resolve(actionFn(produceObj.produce).apply(void 0, arguments)).then(function () {
          return currentStore;
        });
      } catch (e) {
        return Promise.reject(e);
      }
    };
  };

  for (var _i2 = 0, _Object$entries2 = Object.entries((options === null || options === void 0 ? void 0 : options.asyncActions) || {}); _i2 < _Object$entries2.length; _i2++) {
    _loop2();
  }

  var hooksObj = {};

  var _loop3 = function _loop3() {
    var _Object$entries3$_i = _Object$entries3[_i3],
        hookName = _Object$entries3$_i[0],
        hookSelector = _Object$entries3$_i[1];

    hooksObj[hookName] = function (hooksProps, hookPropsDeps) {
      var update = useUpdate(localStoragePlugin);
      var props = react.useMemo(toAnonFunction(hooksProps), hookPropsDeps || [hooksProps]);
      var firstSelectorCall = react.useRef(true);

      var _useMemo = react.useMemo(function () {
        return {
          updateSelector: reselect.createSelector(hookSelector, function (result) {
            stateRef.current = result;

            if (firstSelectorCall.current) {
              firstSelectorCall.current = false;
            } else {
              update();
            }
          }),
          initialStateRef: hookSelector(currentStore, props)
        };
      }, emptyArray),
          updateSelector = _useMemo.updateSelector,
          initialStateRef = _useMemo.initialStateRef;

      var stateRef = react.useRef(initialStateRef);
      useIsomorphicLayoutEffect(function () {
        updateSelector(currentStore, props);
        listeners.set(updateSelector, props);
      }, [props]);
      react.useEffect(function () {
        return function () {
          listeners["delete"](updateSelector);
        };
      }, emptyArray);
      return stateRef.current;
    };
  };

  for (var _i3 = 0, _Object$entries3 = Object.entries((options === null || options === void 0 ? void 0 : options.hooks) || {}); _i3 < _Object$entries3.length; _i3++) {
    _loop3();
  }

  return _extends({
    useStore: useStore,
    actions: _extends({}, actionsObj, asyncActionsObj),
    hooks: hooksObj,
    isReady: isReady
  }, produceObj);
}

var defaultIsReady = /*#__PURE__*/Promise.resolve();
/**
 * Create React Context version of a **react-state-selector** global store
 *
 * @export
 * @template TStore
 * Any object, including array or complex objects like Map, Set or Classes. Even primitives would work.
 * @template THooks
 * Object that defines the custom hooks of the store.
 * @template TActions
 * Object that defines the custom actions of the store.
 * @template TAsyncActions
 * Object that defines the custom async actions of the store.
 * @param {Immutable<TStore>} initialStore
 * Initial state of the store
 * @param {{
 *     hooks?: THooks;
 *     actions?: TActions;
 *     devName?: string;
 *     devToolsInProduction?: boolean;
 *   }} [options]
 * Options object that may contain:
 * hooks: custom hooks;
 * actions: custom actions;
 * devName: name used to activate Redux DevTools;
 * devToolsInProduction: flag to manually activate the usage of the Redux DevTools in production
 * @returns {{
 *   Provider: FunctionComponent<{ debugName?: string }>;
 *   useStore: IUseStore<TStore>;
 *   useProduce: IUseProduce<TStore>;
 *   useActions: () => IActionsObj<TStore, typeof options> & IAsyncActionsObj<TStore, typeof options>;
 *   hooks: IHooksObj<TStore, typeof options>;
 * }}
 * Resulting object containing of the store.
 */

function createStoreContext(initialStore, options) {
  {
    for (var name in options === null || options === void 0 ? void 0 : options.hooks) {
      if (name.length < 4 || name.slice(0, 3) !== "use" || name.charAt(3) === name.charAt(3).toLowerCase()) {
        throw new Error("All hooks should follow the rules of hooks for naming and \"" + name + "\" doesn't");
      }
    }

    if ((options === null || options === void 0 ? void 0 : options.actions) && options.asyncActions) {
      var asyncActionsKeys = Object.keys(options.asyncActions);

      for (var key in options.actions) {
        if (asyncActionsKeys.includes(key)) {
          throw new Error("All the actions and asyncActions should have different names and \"" + key + "\" exists in both objects!");
        }
      }
    }
  }

  var createStoragePersistenceInstance = function createStoragePersistenceInstance(produceStore, debugName) {
    var _options$storagePersi;

    if (options === null || options === void 0 ? void 0 : (_options$storagePersi = options.storagePersistence) === null || _options$storagePersi === void 0 ? void 0 : _options$storagePersi.isActive) {
      if (typeof initialStore !== "object" || Array.isArray(initialStore)) throw new Error("For local storage persistence your store has to be an object");

      var _getPromiseWithCallba = getPromiseWithCallbacks(),
          isReady = _getPromiseWithCallba.promise,
          resolve = _getPromiseWithCallba.resolve,
          reject = _getPromiseWithCallba.reject;

      var persistenceKey;

      if (options.storagePersistence.persistenceKey) {
        persistenceKey = options.storagePersistence.persistenceKey;
      } else if (options.devName) {
        persistenceKey = options.devName;
      }

      if (!persistenceKey) throw new Error("You have to specify persistence key or devName");

      if (debugName) {
        persistenceKey += "-" + debugName;
      } else if (debugName === null) {
        persistenceKey += "-noProvider";
      }

      return Object.assign(connectPersistenceStorage({
        persistenceKey: persistenceKey,
        produce: produceStore,
        debounceWait: options.storagePersistence.debounceWait,
        persistenceMethod: options.storagePersistence.persistenceMethod,
        isSSR: options.storagePersistence.isSSR,
        resolve: resolve,
        reject: reject
      }), {
        isReady: isReady
      });
    }

    return null;
  };

  var createDevToolsInstance = function createDevToolsInstance(debugName) {
    if ((options === null || options === void 0 ? void 0 : options.devName) && (options.devToolsInProduction || Constants.IS_NOT_PRODUCTION)) {
      var devToolsName;

      if (debugName) {
        devToolsName = options.devName + "-" + debugName;
      } else if (debugName === null) {
        devToolsName = options.devName + "-noProvider";
      } else {
        devToolsName = options.devName;
      }

      var devTools = connectDevTools(devToolsName);

      if (devTools) {
        devTools.init(initialStore);
      }

      return devTools;
    }

    return undefined;
  };

  var createProduceObj = function createProduceObj(storeRef) {
    return {
      produce: function produce(draft) {
        var _storeRef$storagePers;

        if (typeof draft !== "function") return storeRef.store;

        if (storeRef.devTools) {
          var produceFn = immer.produceWithPatches(draft);
          var produceResult = produceFn(storeRef.store);
          storeRef.store = produceResult[0];
          storeRef.devTools.send({
            type: "produce",
            payload: produceResult[1]
          }, storeRef.store);
        } else {
          var _produceFn = immer.produce(draft);

          storeRef.store = _produceFn(storeRef.store);
        }

        storeRef.listeners.forEach(function (props, listener) {
          listener(storeRef.store, props);
        });
        (_storeRef$storagePers = storeRef.storagePersistence) === null || _storeRef$storagePers === void 0 ? void 0 : _storeRef$storagePers.setState(storeRef.store);
        return storeRef.store;
      },
      asyncProduce: function (draft) {
        try {
          if (typeof draft !== "function") return Promise.resolve(storeRef.store);
          var storeDraft = immer.createDraft(storeRef.store);
          return Promise.resolve(Promise.resolve(draft(storeDraft))).then(function () {
            var _storeRef$storagePers2;

            immer.finishDraft(storeDraft, function (changes) {
              if (changes.length) {
                storeRef.store = immer.applyPatches(storeRef.store, changes);

                if (storeRef.devTools) {
                  storeRef.devTools.send({
                    type: "asyncProduce",
                    payload: changes
                  }, storeRef.store);
                }

                storeRef.listeners.forEach(function (props, listener) {
                  listener(storeRef.store, props);
                });
              }
            });
            (_storeRef$storagePers2 = storeRef.storagePersistence) === null || _storeRef$storagePers2 === void 0 ? void 0 : _storeRef$storagePers2.setState(storeRef.store);
            return storeRef.store;
          });
        } catch (e) {
          return Promise.reject(e);
        }
      }
    };
  };

  var providerRefCallback = function providerRefCallback(debugName) {
    return function () {
      var storeRef = {
        store: initialStore,
        listeners: new Map(),
        devTools: createDevToolsInstance(debugName)
      };
      var isReady = defaultIsReady;
      var produceObj = createProduceObj(storeRef);
      storeRef.produce = produceObj;
      var persistance = createStoragePersistenceInstance(produceObj.produce, debugName);

      if (persistance) {
        var current = persistance.current,
            getState = persistance.getState,
            setState = persistance.setState;
        storeRef.storagePersistence = {
          current: current,
          getState: getState,
          setState: setState
        };
        isReady = persistance.isReady;
      }

      return Object.assign(storeRef, {
        produce: produceObj,
        isReady: isReady
      });
    };
  };

  var StoreContext = react.createContext({
    current: providerRefCallback(null)()
  });
  var Provider = react.memo(function (_ref) {
    var children = _ref.children,
        debugName = _ref.debugName;
    var initialRef = react.useMemo(providerRefCallback(debugName), emptyArray);
    var value = react.useRef(initialRef);
    return react.createElement(StoreContext.Provider, {
      value: value,
      children: children
    });
  });

  var useStore = function useStore() {
    var storeCtx = react.useContext(StoreContext);
    var update = useUpdate(storeCtx.current.storagePersistence);
    useIsomorphicLayoutEffect(function () {
      var globalListener = reselect.createSelector(function (s) {
        return s;
      }, function () {
        update();
      });
      storeCtx.current.listeners.set(globalListener, null);
      return function () {
        storeCtx.current.listeners["delete"](globalListener);
      };
    }, emptyArray);
    return storeCtx.current.store;
  };

  var useProduce = function useProduce() {
    var storeCtx = react.useContext(StoreContext);
    return storeCtx.current.produce;
  };

  var useActions = function useActions() {
    var storeCtx = react.useContext(StoreContext);
    var produceObj = useProduce();
    return react.useMemo(function () {
      if (storeCtx.current.actions === undefined) {
        var actionsObj = {};
        var asyncActionsObj = {};

        var _loop = function _loop() {
          var _Object$entries$_i = _Object$entries[_i],
              actionName = _Object$entries$_i[0],
              actionFn = _Object$entries$_i[1];

          actionsObj[actionName] = function () {
            for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
              args[_key] = arguments[_key];
            }

            var actionDraft = actionFn.apply(void 0, args.concat([produceObj.produce]));

            if (storeCtx.current.devTools) {
              var produceFn = immer.produceWithPatches(actionDraft);
              var produceResult = produceFn(storeCtx.current.store);
              storeCtx.current.store = produceResult[0];
              storeCtx.current.devTools.send({
                type: actionName,
                payload: produceResult[1]
              }, storeCtx.current.store);
            } else {
              var _produceFn2 = immer.produce(actionDraft);

              storeCtx.current.store = _produceFn2(storeCtx.current.store);
            }

            storeCtx.current.listeners.forEach(function (props, listener) {
              listener(storeCtx.current.store, props);
            });
            return storeCtx.current.store;
          };
        };

        for (var _i = 0, _Object$entries = Object.entries((options === null || options === void 0 ? void 0 : options.actions) || {}); _i < _Object$entries.length; _i++) {
          _loop();
        }

        var _loop2 = function _loop2() {
          var _Object$entries2$_i = _Object$entries2[_i2],
              actionName = _Object$entries2$_i[0],
              actionFn = _Object$entries2$_i[1];

          asyncActionsObj[actionName] = function () {
            try {
              return Promise.resolve(actionFn(produceObj.produce).apply(void 0, arguments)).then(function () {
                return storeCtx.current.store;
              });
            } catch (e) {
              return Promise.reject(e);
            }
          };
        };

        for (var _i2 = 0, _Object$entries2 = Object.entries((options === null || options === void 0 ? void 0 : options.asyncActions) || {}); _i2 < _Object$entries2.length; _i2++) {
          _loop2();
        }

        storeCtx.current.actions = _extends({}, actionsObj, asyncActionsObj);
      }

      return storeCtx.current.actions;
    }, emptyArray);
  };

  var hooksObj = {};

  var _loop3 = function _loop3() {
    var _Object$entries3$_i = _Object$entries3[_i3],
        hookName = _Object$entries3$_i[0],
        hookSelector = _Object$entries3$_i[1];

    hooksObj[hookName] = function (hooksProps, hookPropsDeps) {
      var storeCtx = react.useContext(StoreContext);
      var update = useUpdate(storeCtx.current.storagePersistence);
      var props = react.useMemo(toAnonFunction(hooksProps), hookPropsDeps || [hooksProps]);
      var firstSelectorCall = react.useRef(true);

      var _useMemo = react.useMemo(function () {
        return {
          updateSelector: reselect.createSelector(hookSelector, function (result) {
            stateRef.current = result;

            if (firstSelectorCall.current) {
              firstSelectorCall.current = false;
            } else {
              update();
            }
          }),
          initialStateRef: hookSelector(storeCtx.current.store, props)
        };
      }, emptyArray),
          updateSelector = _useMemo.updateSelector,
          initialStateRef = _useMemo.initialStateRef;

      var stateRef = react.useRef(initialStateRef);
      useIsomorphicLayoutEffect(function () {
        updateSelector(storeCtx.current.store, props);
        storeCtx.current.listeners.set(updateSelector, props);
      }, [props]);
      react.useEffect(function () {
        return function () {
          storeCtx.current.listeners["delete"](updateSelector);
        };
      }, emptyArray);
      return stateRef.current;
    };
  };

  for (var _i3 = 0, _Object$entries3 = Object.entries((options === null || options === void 0 ? void 0 : options.hooks) || {}); _i3 < _Object$entries3.length; _i3++) {
    _loop3();
  }

  var useIsReady = function useIsReady() {
    var _useContext = react.useContext(StoreContext),
        isReady = _useContext.current.isReady;

    var _useState = react.useState(isReady === defaultIsReady),
        ready = _useState[0],
        setReady = _useState[1];

    react.useEffect(function () {
      if (isReady !== defaultIsReady) {
        isReady.then(function () {
          setTimeout(function () {
            setReady(true);
          }, 0);
        });
      }
    }, [setReady]);
    return ready;
  };

  return {
    Provider: Provider,
    useStore: useStore,
    useProduce: useProduce,
    useActions: useActions,
    hooks: hooksObj,
    useIsReady: useIsReady
  };
}

Object.keys(immer).forEach(function (k) {
  if (k !== 'default') Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () {
      return immer[k];
    }
  });
});
Object.defineProperty(exports, 'createSelector', {
  enumerable: true,
  get: function () {
    return reselect.createSelector;
  }
});
exports.createStore = createStore;
exports.createStoreContext = createStoreContext;
//# sourceMappingURL=react-state-selector.cjs.development.js.map
